﻿using System;

namespace DecoratedBook
{
    public class ThreadSafeBook : IBook
    {
        private readonly IBook _book;

        public ThreadSafeBook(IBook book)
        {
            if (book == null) throw new ArgumentNullException("book");
            _book = book;
        }

        public string Author
        {
            get { lock (this) { return _book.Author; } }
            set { lock (this) { _book.Author = value; } }
        }

        public string Title
        {
            get { lock (this) { return _book.Title; } }
            set { lock (this) { _book.Title = value; } }
        }

        public int YearOfPublication
        {
            get { lock (this) { return _book.YearOfPublication; } }
            set { lock (this) { _book.YearOfPublication = value; } }
        }

        public override string ToString()
        {
            lock (this)
            {
                return _book.ToString();
            }
        }

        public override bool Equals(object obj)
        {
            lock (this) { 
            return _book.Equals(obj);
            }
        }

        public override int GetHashCode()
        {
            lock (this) {
                return _book.GetHashCode();
            }
        }
    }
}
