﻿using System;
using DecoratedBook;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace DecoratedBookTests1
{
    [TestClass]
    public class UnmodifiableBookTests
    {
        [TestMethod]
        public void UnmodifiableBookTest()
        {
            try
            {
                new UnmodifiableBook(null);
                Assert.Fail();
            } catch (ArgumentNullException) {  /* empty */ }
        }

        [TestMethod]
        public void AuthorTest()
        {
            IBook ordinaryBook = new Book
            {
                Author = "Sturm",
                Title = "Functional Programming in C#",
                YearOfPublication = 2011
            };
            IBook unmodifiableBook = new UnmodifiableBook(ordinaryBook);
            Assert.AreEqual("Sturm", unmodifiableBook.Author);
            Assert.AreEqual("Functional Programming in C#", unmodifiableBook.Title);
            Assert.AreEqual(2011, unmodifiableBook.YearOfPublication);
            try
            {
                unmodifiableBook.Author = "Anders";
                Assert.Fail();
            }
            catch (NotSupportedException) { /* empty */ }

            try
            {
                unmodifiableBook.Title = "My Own Book";
                Assert.Fail();
            }
            catch (NotSupportedException) { /* empty */ }

            try
            {
                unmodifiableBook.YearOfPublication = 2014;
                Assert.Fail();
            }
            catch (NotSupportedException) { /* empty */ }

        }

        [TestMethod]
        public void EqualsHashcodeTest()
        {
            IBook ordinaryBook = new Book
            {
                Author = "Sturm",
                Title = "Functional Programming in C#",
                YearOfPublication = 2011
            };
            IBook unmodifiableBook = new UnmodifiableBook(ordinaryBook);
            Assert.IsTrue(ordinaryBook.Equals(unmodifiableBook));
            Assert.IsTrue(unmodifiableBook.Equals(ordinaryBook));
            Assert.AreEqual(ordinaryBook.GetHashCode(), unmodifiableBook.GetHashCode());
            Assert.AreEqual(unmodifiableBook.GetHashCode(), ordinaryBook.GetHashCode());
        }
    }
}
